#include <iostream>
// Ukljucujemo zaglavlje algorithm kako bismo mogli da koristimo algoritme standardne biblioteke
#include <algorithm>
#include <vector>

// Ovakve strukture se nazivaju funkcijski objekti, oni predefinisu operator () i omogucavaju poredjenja. Povratna vrednost ovog operatora je bool
struct greater_than
{
public:
    // Konstruktor koji nam omogucava da damo vrednost sa kojom zelimo da vrsimo poredjenja
    // this je pokazivac pa zato imamo ->
    greater_than(int x) { this->x = x; }

    // Predefinisemo operator (), on kao argument dobija neku vrednost a koju poredimo sa x, tj proveravamo da li je a vece od x
    bool operator() (int a) { return a > x; }

private:
    int x;
};

// Funkcija koja dobija vrednost x i vraca true ako je x pozitivno a false inace
bool pozitivni(int x)
{
    return x > 0;
}

// Algoritam std::all_of proverava da li svi elementi kolekcije zadovoljavaju predikat (uslov) koji se navodi kao 3. argument. Predikat moze biti posebno napisana funkcija ili moze biti lambda funkcija
// Lambda funkcija je anonimna funkcija (nema naziv) koja se u algoritmu izvrsava za svaki element kolekcije
// Njena sintaksa je [argumenti koji se vezuju od spolja, ne navodi se tip vec samo promenljiva koja se vezuje](argument kolekcije, navodi se i tip promenljive){telo lambde gde se izvrsavaju neke operacije i vraca odgovarajuca vrednost}
void all_of(const std::vector<int> &a)
{
    // Ovde proveravamo da li su svi elementi vektora pozitivni pomocu lambde, [] => ne vezujemo nijednu promenljivu, (int x) => svaki od elemenata kolekcije ce biti int, {return x > 0;} => za svaki od elemenata proveravamo da li je pozitivan ili ne
    std::cout << "Lambda pozitivni: " << std::all_of(a.begin(), a.end(), [](int x){return x > 0;}) << std::endl;

    // Kao 3. argument saljemo pokazivac na funkciju koja nam daje odgovor na pitanje da li je broj pozitivan
    std::cout << "Funkcija pozitivni: " << std::all_of(a.begin(), a.end(), pozitivni) << std::endl;
}

// Proveravamo da li su svi elementi kolekcije veci od value
void greater_all_of(const std::vector<int> &a, int value)
{
    // Ovde proveravamo da li su svi elementi vektora veci od argumenta value pomocu lambde, [value] => vezujemo promenljivu value jer hocemo da bude vidljiva unutar lambde, da je nismo vezali lambda ne bi znala da ona postoji i ne bismo mogli da je koristimo, (int x) => svaki od elemenata kolekcije ce biti int, {return x > value;} => za svaki od elemenata proveravamo da li je veci od value ili ne
    std::cout << "Lambda veci od: " << std::all_of(a.begin(), a.end(), [value](int x){return x > value;}) << std::endl;

    // Mozemo da kreiramo strukturu koja implementira () i koja ce nam vrsiti trazeno poredjenje, mozemo i prilikom koriscenja algoritma da kao treci argument kreiramo strukturu sa vrednoscu value koja ce nam vrsiti poredjenje
    greater_than gt(value);

    // Kao 3. argument funkcijski objekat koji ce vrsiti odgovarajuce poredjenje
    std::cout << "Funkcijski objekat veci od: " << std::all_of(a.begin(), a.end(), gt) << std::endl;
    
    // Otkomentarisati narednu liniju i videti da je ponasanje isto
    //std::cout << "Funkcijski objekat veci od: " << std::all_of(a.begin(), a.end(), greater_than(value)) << std::endl;
}

int main ()
{
    std::vector<int> a = {1,2,3,4,5,6,7,8,9};
    
    std::vector<int> a1 = {1,-2,3,-4,5,-6,7,-8,9};
    
    all_of(a);

    all_of(a1);

    greater_all_of(a, 0);

    greater_all_of(a1, 0);

    return 0;
}   
